SystemOrganization addCategory: #TusLibros!

TestCase subclass: #CashierTest
	instanceVariableNames: 'catalog calendar expiredCard validCard paymentTerminal cashier'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!CashierTest methodsFor: 'running' stamp: 'JuanEdi 6/2/2013 01:07'!
setUp

	catalog := Dictionary new.
	catalog at: 'Harry Potter' put: 10.
	catalog at: 'Dracula' put: 15.
	
	calendar := FixedCalendar setIn: (Date newDay: 1 month: 2 year: 2000 ).
	
	expiredCard := Card withNumber: 123 expiring: (Month month: 2 year: 2000).
	validCard := Card withNumber: 456 expiring: (Month month: 3 year: 2000).
	
	paymentTerminal := PaymentTerminalStub new.
	
	cashier := Cashier usingCatalog: catalog paymentTerminal: paymentTerminal andCalendar: calendar.
! !

!CashierTest methodsFor: 'checkout' stamp: 'JuanEdi 6/2/2013 01:13'!
testShouldAddTransactionWhenCheckoutValidCart
	
	| aCart initialTransactionCount |
	
	initialTransactionCount := paymentTerminal transactionCount.
	aCart := ShoppingCart newWithCatalog: catalog.
	aCart add: 'Harry Potter'.
	
	cashier checkout: aCart usingCard: validCard.
	
	self assert: paymentTerminal transactionCount = (initialTransactionCount + 1).! !

!CashierTest methodsFor: 'checkout' stamp: 'JuanEdi 6/2/2013 01:13'!
testShouldDebitTheCartAmountWhenCheckout
	| aCart expectedAmount |
	
	aCart := ShoppingCart newWithCatalog: catalog.
	aCart add: 'Harry Potter' withOcurrences: 2.
	aCart add: 'Dracula' withOcurrences: 3.
	
	cashier checkout: aCart usingCard: validCard.
	
	expectedAmount := (2 * (catalog at: 'Harry Potter')) +  (3 * catalog at: 'Dracula').
	
	self assert: (paymentTerminal lastCardUsed) = validCard.
	self assert: (paymentTerminal lastAmountProcessed) = expectedAmount.! !

!CashierTest methodsFor: 'checkout' stamp: 'JuanEdi 6/2/2013 01:08'!
testShouldFailIfTryingToCheckoutEmptyCart
	
	| initialTransactionCount |
	
	initialTransactionCount := paymentTerminal transactionCount.
	
	self should: [ cashier checkout: (ShoppingCart newWithCatalog: catalog) usingCard: '12345' ]
		  raise: Error
		  withExceptionDo: [ :anException | 
			self assert: anException messageText = Cashier emptyCartCheckoutMessage. 
			self assert: paymentTerminal transactionCount = initialTransactionCount.
		  ]! !

!CashierTest methodsFor: 'checkout' stamp: 'JuanEdi 6/2/2013 01:14'!
testShouldFailIfTryingToCheckoutWithExpiredCreditCard
	
	| aCart initialTransactionCount |
	
	initialTransactionCount := paymentTerminal transactionCount.
	
	aCart := ShoppingCart newWithCatalog: catalog.
	aCart add: 'Harry Potter'.
		
	self should: [ cashier checkout: aCart usingCard: expiredCard ]
		  raise: Error
		  withExceptionDo: [ :anException | 
			self assert: anException messageText = Cashier expiredCardCheckoutMessage. 
			self assert: paymentTerminal transactionCount = initialTransactionCount.
		  ]! !

TestCase subclass: #ShoppingCartTest
	instanceVariableNames: 'catalog calendar expiredCard validCard'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!ShoppingCartTest commentStamp: '<historical>' prior: 0!
A ShoppingCartTest is a test class for testing the behavior of ShoppingCart!

!ShoppingCartTest methodsFor: 'running' stamp: 'JuanEdi 6/2/2013 01:06'!
setUp

	catalog := Dictionary new.
	catalog at: 'Harry Potter' put: 10.
	catalog at: 'Dracula' put: 15.! !

!ShoppingCartTest methodsFor: 'cart usage' stamp: 'JuanEdi 6/1/2013 22:54'!
testCartShouldBeEmptyAfterCreation

	self assert: (ShoppingCart newWithCatalog: catalog) isEmpty.! !

!ShoppingCartTest methodsFor: 'cart usage' stamp: 'JuanEdi 6/1/2013 22:50'!
testShouldContainBookAfterAddingItToACart
	| cart |
	cart := ShoppingCart newWithCatalog: catalog.
	cart add: 'Dracula' withOcurrences: 2.
	self assert: (cart occurrencesOf: 'Dracula') = 2.
	self assert: (cart itemCount) = 2.! !

!ShoppingCartTest methodsFor: 'cart usage' stamp: 'JuanEdi 6/1/2013 22:51'!
testShouldFailIfTryingToAddANegativeAmountOfProducts

	| cart |
	cart := ShoppingCart newWithCatalog: catalog.
	cart add: 'Harry Potter' withOcurrences: 2.
	self should: [ cart add: 'Harry Potter' withOcurrences: -1 ] 
			raise: Error 
			withExceptionDo: [ :anException | self assert: anException messageText = ShoppingCart invalidAmountOfProductsMessage ].
	self assert: (cart occurrencesOf: 'Harry Potter') = 2.
	self assert: (cart itemCount) = 2.
	! !

!ShoppingCartTest methodsFor: 'cart usage' stamp: 'JuanEdi 6/1/2013 22:52'!
testShouldOnlyAcceptExistingBooks
	| cart |
	cart := ShoppingCart newWithCatalog: catalog.
	self should: [ cart add: 'Cien aos de soledad' withOcurrences: 2 ] 
			raise: Error 
			withExceptionDo: [ :anException | self assert: anException messageText = ShoppingCart outOfCatalogMessage ].
	self assert: cart isEmpty.
	! !

!ShoppingCartTest methodsFor: 'cart usage' stamp: 'JuanEdi 6/1/2013 22:52'!
testShouldUpdateBookOrderCountAfterAddingMoreCopies
	| cart |
	cart := ShoppingCart newWithCatalog: catalog.
	cart add: 'Dracula' withOcurrences: 2 .
	cart add: 'Dracula' withOcurrences: 3.
	self assert: (cart occurrencesOf: 'Dracula') = 5.
	self assert: (cart itemCount) = 5.! !

Object subclass: #Calendar
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Calendar methodsFor: 'accessing' stamp: 'JuanEdi 6/2/2013 00:55'!
currentMonth

	^ Date today month.! !

Object subclass: #Card
	instanceVariableNames: 'number expirationMonth'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Card class methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/2/2013 00:22'!
withNumber: aCardNumber expiring: aMonth 
	
	^ self new initializeWithNumber: aCardNumber expiring: aMonth.! !

!Card methodsFor: 'accessing' stamp: 'JuanEdi 6/2/2013 00:35'!
expirationMonth
	
	^ expirationMonth! !

!Card methodsFor: 'initialize-release' stamp: 'JuanEdi 6/2/2013 00:22'!
initializeWithNumber: aCardNumber expiring: aMonth 
	
	number := aCardNumber.
	expirationMonth := aMonth.! !

Object subclass: #Cashier
	instanceVariableNames: 'paymentTerminal catalog calendar'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Cashier class methodsFor: 'as yet unclassified' stamp: 'JuanEdi 5/29/2013 20:33'!
emptyCartCheckoutMessage
	
	^ 'Cannot checkout empty carts.'! !

!Cashier class methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/2/2013 00:23'!
expiredCardCheckoutMessage
	
	^ 'Cannot checkout a cart with an expired card'! !

!Cashier class methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/2/2013 00:31'!
usingCatalog: aCatalog paymentTerminal: aPaymentTerminal andCalendar: aCalendar
	
	^ self new usingCatalog: aCatalog paymentTerminal: aPaymentTerminal andCalendar: aCalendar.! !

!Cashier methodsFor: 'checkout' stamp: 'JuanEdi 6/2/2013 01:00'!
checkout: aCart usingCard: aCard
	| amountToDebit |
	self validateCartIsNotEmpty: aCart.
	self validateCardIsNotExpired: aCard.
	
	amountToDebit := 0.
	aCart do: [ :anItem | amountToDebit := amountToDebit + (catalog at: anItem) ].
	paymentTerminal debit: amountToDebit from: aCard.! !

!Cashier methodsFor: 'initialize-release' stamp: 'JuanEdi 6/2/2013 00:32'!
usingCatalog: aCatalog paymentTerminal: aPaymentTerminal andCalendar: aCalendar
	
	catalog := aCatalog.
	paymentTerminal := aPaymentTerminal.
	calendar := aCalendar.! !

!Cashier methodsFor: 'validating' stamp: 'JuanEdi 6/2/2013 00:49'!
validateCardIsNotExpired: aCard 
	
	((aCard expirationMonth) <= (calendar currentMonth)) ifTrue: [self error: Cashier expiredCardCheckoutMessage]! !

!Cashier methodsFor: 'validating' stamp: 'JuanEdi 6/2/2013 01:00'!
validateCartIsNotEmpty: aShoppingCart 
	aShoppingCart isEmpty ifTrue: [self error: Cashier emptyCartCheckoutMessage]! !

Object subclass: #FixedCalendar
	instanceVariableNames: 'date'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!FixedCalendar commentStamp: '<historical>' prior: 0!
A FixedCalendar is useful when there is a need to fix the date used as the current date during a test.

Instance Variables
	date:		<Date>

date
	- xxxxx
!

!FixedCalendar class methodsFor: 'as yet unclassified' stamp: 'JuanEdi 6/2/2013 00:38'!
setIn: aDate

	^ self new setIn: aDate.! !

!FixedCalendar methodsFor: 'accessing' stamp: 'JuanEdi 6/2/2013 00:37'!
currentMonth

	^ date month.! !

!FixedCalendar methodsFor: 'initialize-release' stamp: 'JuanEdi 6/2/2013 00:44'!
setIn: aDate

	date := aDate.! !

Object subclass: #PaymentTerminalStub
	instanceVariableNames: 'transactionCount lastCardUsed lastAmountProcessed'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!PaymentTerminalStub methodsFor: 'transaction processing' stamp: 'JuanEdi 6/1/2013 22:03'!
debit: anAmount from: aCard 
	transactionCount := transactionCount + 1.
	lastCardUsed  := aCard.
	lastAmountProcessed := anAmount.! !

!PaymentTerminalStub methodsFor: 'initialize-release' stamp: 'JuanEdi 5/29/2013 21:17'!
initialize

	transactionCount := 0.! !

!PaymentTerminalStub methodsFor: 'accessing' stamp: 'JuanEdi 6/1/2013 22:36'!
lastAmountProcessed
	
	^ lastAmountProcessed! !

!PaymentTerminalStub methodsFor: 'accessing' stamp: 'JuanEdi 6/1/2013 22:02'!
lastCardUsed
	
	^ lastCardUsed.! !

!PaymentTerminalStub methodsFor: 'accessing' stamp: 'JuanEdi 5/29/2013 21:16'!
transactionCount
	
	^ transactionCount! !

Object subclass: #ShoppingCart
	instanceVariableNames: 'catalog products'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!ShoppingCart class methodsFor: 'as yet unclassified' stamp: 'JuanEdi 5/29/2013 20:23'!
invalidAmountOfProductsMessage
	
	^ 'Amount of products to add should be greater than zero'.! !

!ShoppingCart class methodsFor: 'instance-creation' stamp: 'JuanEdi 5/27/2013 21:30'!
newWithCatalog: aBookCatalog 
	
	^ self new initializeWithCatalog: aBookCatalog.! !

!ShoppingCart class methodsFor: 'exception messages' stamp: 'JuanEdi 5/29/2013 20:07'!
outOfCatalogMessage
	^ 'Error adding product to cart, the product is not in the catalog.'! !

!ShoppingCart methodsFor: 'adding' stamp: 'JuanEdi 6/1/2013 21:38'!
add: aProduct 
	self add: aProduct withOcurrences: 1.! !

!ShoppingCart methodsFor: 'adding' stamp: 'JuanEdi 6/1/2013 22:27'!
add: aProduct withOcurrences: aNumber 
	
	self validatePossitiveAmountOfProducts: aNumber.
	self validateCatalogContainsProduct: aProduct.
		
	products add: aProduct withOccurrences: aNumber.! !

!ShoppingCart methodsFor: 'enumerating' stamp: 'JuanEdi 6/1/2013 22:54'!
do: aBlock
	"Itera sobre cada ocurrencia individual de cada producto del carro."

	products do: aBlock! !

!ShoppingCart methodsFor: 'initialize-release' stamp: 'JuanEdi 6/1/2013 22:24'!
initializeWithCatalog: aCatalog 

	catalog := aCatalog.
	products := Bag new.! !

!ShoppingCart methodsFor: 'inspecting' stamp: 'JuanEdi 5/29/2013 19:50'!
isEmpty

	^ products isEmpty.! !

!ShoppingCart methodsFor: 'accessing' stamp: 'JuanEdi 6/1/2013 22:31'!
itemCount
	"Retorna la cantidad de items en total, contando todas las unidades de cada item agregado."
	^ products size.! !

!ShoppingCart methodsFor: 'inspecting' stamp: 'JuanEdi 6/1/2013 22:55'!
occurrencesOf: aProduct 
	
	^ products occurrencesOf: aProduct.! !

!ShoppingCart methodsFor: 'validations' stamp: 'JuanEdi 6/1/2013 22:34'!
validateCatalogContainsProduct: aProduct

	(catalog keys includes: aProduct) ifFalse: [ self error: ShoppingCart outOfCatalogMessage]! !

!ShoppingCart methodsFor: 'validations' stamp: 'JuanEdi 5/29/2013 20:22'!
validatePossitiveAmountOfProducts: anAmount

	(anAmount > 0) ifFalse: [ self error: ShoppingCart invalidAmountOfProductsMessage]! !
